import React from "react";

type ManagedFormElement<T = string> = {
  onChange: (v: T) => void;
  nullable?: false;
  value: T;
};

type UnmanagedFormElement = {
  nullable?: boolean;
  onChange?: never;
  value?: never;
};

type NullableManagedFormElement<T = string> = {
  onChange: (v: T) => void;
  value: T | null;
  nullable: true;
};

type ValidatedFormElement<T = string> = {
  validation: ((value: T) => string) | ((value: T) => boolean) | RegExp;
  validationMessage: string;
  validationMode: "always" | "interaction";
};

type NonValidatedFormElement = {
  validation?: never;
  validationMessage?: never;
  validationMode?: never;
};

type BaseFormElement = {
  className?: string;
  disabled?: boolean;
  label?: React.ReactNode;
  placeholder?: string;
};

export type FormElement<T = string> = BaseFormElement &
  (
    | ManagedFormElement<T>
    | UnmanagedFormElement
    | NullableManagedFormElement<T>
  ) &
  (ValidatedFormElement<T> | NonValidatedFormElement);
