import React from "react";
import { makeStringValidator, StringValidatorLike, Validator } from "@devcake/utilities";

type UseValidationType = (
  validation: StringValidatorLike,
  message?: string | null,
  interaction?: boolean,
  nullable?: boolean,
  initial?: string
) => {
  error: string | null;
  value: string | null;
  interact: () => void;
  setValue: (v: string) => void;
}

export const useValidation: UseValidationType = (validation, message, interaction = true, nullable = true, initial = null) => {
  if (validation === undefined) validation = () => true;

  const validator = React.useMemo<Validator<string>>(() => {
    const validator = makeStringValidator(validation, message);
    return value => {
      if (!nullable && (value === "" || value === null)) return message || "Field may not be empty";
      return validator(value);
    };
  }, [validation, message]);
  const [hadInteraction, setInteraction] = React.useState(!interaction);
  const [value, setValue] = React.useState<string>(initial);
  const [error, setError] = React.useState<string>(null);

  return {
    error,
    value,
    interact: () => setInteraction(true),
    setValue: value => {
      if (nullable && value === "") value = null;
      if (!nullable && value === null) value = "";
      setValue(value);
      if (hadInteraction) {
        setError(validator(value));
      }
    }
  };
};
